@@ -229,14 +229,14 @@ Projection.prototype.postTransform = function(entity) {
  * Callback function for Transform.getOperation. Generic for all projections.
  * \param k Ignored
  */
-Projection.prototype.transform = function(entity, k, op, preview, forceNew) {
+Projection.prototype.transform = function(entity, k, op, preview, flags) {
     var i;
 
     this.preTransform(entity);
 
     // shapes that represent this entity:
     var shapes = entity.getShapes();
-    this.addTransformedShapes(entity, shapes, op, preview, forceNew);
+    this.addTransformedShapes(entity, shapes, op, preview, flags);
 
     // text data that is part of this entity (dimension label):
     if (isDimensionEntity(entity) && isFunction(entity.getTextData)) {
@@ -247,7 +247,7 @@ Projection.prototype.transform = function(entity, k, op, preview, forceNew) {
                 continue;
             }
             shapes = painterPaths[i].getShapes();
-            this.addTransformedShapes(entity, shapes, op, preview, forceNew);
+            this.addTransformedShapes(entity, shapes, op, preview, flags);
         }
     }
 };
@@ -260,10 +260,36 @@ Projection.prototype.transform = function(entity, k, op, preview, forceNew) {
  * \param shapes Array The transformed shapes
  * \param op RAddObjectsOperation Operation
  */
-Projection.prototype.addTransformedShapes = function(entity, shapes, op, preview, forceNew) {
-    for (var i=0; i<shapes.length; i++) {
-        var s = this.projectShape(shapes[i].data(), preview);
-        for (var n=0; n<s.length; n++) {
+Projection.prototype.addTransformedShapes = function(entity, shapes, op, preview, flags) {
+    var i, s, n;
+
+    if (isHatchEntity(entity) && entity.isSolid()) {
+        var hd = entity.getData();
+        hd.clearBoundary();
+        hd.newLoop();
+
+        for (i=0; i<shapes.length; i++) {
+            s = this.projectShape(shapes[i].data(), preview);
+            for (n=0; n<s.length; n++) {
+                if (isNull(s[n])) {
+                    debugger;
+                }
+
+                hd.addBoundary(s[n], true);
+            }
+        }
+
+        var e = entity.clone();
+        e.setData(hd);
+
+        flags = flags | RAddObjectsOperation.UseAttributes;
+        op.addObject(e, flags);
+        return;
+    }
+
+    for (i=0; i<shapes.length; i++) {
+        s = this.projectShape(shapes[i].data(), preview);
+        for (n=0; n<s.length; n++) {
             if (isNull(s[n])) {
                 debugger;
             }
@@ -275,7 +301,8 @@ Projection.prototype.addTransformedShapes = function(entity, shapes, op, preview
 
             e.copyAttributesFrom(entity);
             //e.move(this.targetPoint);
-            op.addObject(e, false, forceNew);
+            flags = flags | RAddObjectsOperation.UseAttributes;
+            op.addObject(e, flags);
         }
     }
 };
@@ -353,6 +380,10 @@ Projection.prototype.projectShape = function(shape, preview, trim) {
             if (!preview && this.segmentLength>RS.PointTolerance) {
                 numSegments = Math.ceil(l/this.segmentLength);
             }
+            if (numSegments>1000) {
+                // avoid huge number of segments / crash:
+                numSegments = 1000;
+            }
 
             var polyline = ShapeAlgorithms.lineOrArcToPolyline(shape, numSegments);
             var projectedPl = this.projectShape(polyline, preview, trim)[0];
@@ -483,6 +514,9 @@ Projection.prototype.projectShape = function(shape, preview, trim) {
             // iterate through projected shapes (trimming might lead to gaps):
             for (k=0; k<s.length; k++) {
                 var seg = s[k];
+                if (isFunction(seg.data)) {
+                    seg = seg.data();
+                }
 
                 var gotGap = pl.countVertices()>0 &&
                     !pl.getEndPoint().equalsFuzzy(seg.getStartPoint());
@@ -513,6 +547,7 @@ Projection.prototype.projectShape = function(shape, preview, trim) {
                     }
                     else if (gotGap){
                         pl = new RPolyline();
+                        //qDebug("seg:", seg);
                         pl.appendShape(seg);
                         addPl = true;
                     }
